unit Rtcbdlg;

{ RTComboDlg                                                    }
{ Copyright 1996 R&T Software, Tomasz Stanczak                  }
{ CIS 100735,3273                                               }
{ info@rtsoftware.com                                           }
{ WWW http://www.rtsoftware.com                                 }
{                                                               }
{ TRTComboDlg is simply a MaskEdit with a SpeedButton inside,   }
{ on pressing this button a OnDialog event is fired, so that    }
{ you can show whatever you need and set its as you wish.       }
{                                                               }
{ Actually that's all I can say about it, it's really very      }
{ simple, but I have found it quite useful at times.            }
{                                                               }
{ This component is a freeware, so you're granted an unlimited  }
{ license to use it for anything you want, except that you're   }
{ not allowed to distribute and/or market it as your own work,  }
{ alone or as a part of any component or component libraries in }
{ source code or compiled DCUs.                                 }
{                                                               }
{ All rights not explicitly granted here are reserved by the    }
{ author.                                                       }
{                                                               }
{ There are no warranties implied or explicite. You cannot make }
{ me responsible for anything you do with this component and I  }
{ won't promise to support it.                                  }

interface

uses
	WinTypes, WinProcs, Messages, SysUtils, Classes, Controls, Menus,
   StdCtrls, Buttons, Graphics, Mask;

type
   TRtComboDlgStyle = (csEditable, csDialog);

   TRtCustomComboDlg = class(TCustomMaskEdit)
	private
		FAlignment: TAlignment;
		FWidth: integer;
      FButton: TSpeedButton;
      FStyle: TRtComboDlgStyle;
		FOnDialog: TNotifyEvent;
		procedure SetAlignment(Value: TAlignment);
      procedure SetStyle(Value: TRtComboDlgStyle);
		procedure SetEditRect;
      function GetShowButton: Boolean;
		procedure SetShowButton(Value: Boolean);
		function GetEnableButton: Boolean;
		procedure SetEnableButton(Value: Boolean);
      function GetFont: TFont;
      procedure SetFont(Value: TFont);
		function StoreFont: Boolean;
      function GetCaption: string;
      procedure SetCaption(Value: string);
      function GetGlyph: TBitmap;
      procedure SetGlyph(Value: TBitmap);
		function GetNumGlyphs: TNumGlyphs;
		procedure SetNumGlyphs(Value: TNumGlyphs);
      function GetWidth: integer;
      procedure SetWidth(Value: integer);
		function GetMinHeight: integer;
      procedure WMPaste (var Message: TMessage); message WM_PASTE;
      procedure WMCut (var Message: TMessage); message WM_CUT;
      procedure WMSize(var Message: TWMSize); message WM_SIZE;
      procedure CMExit(var Message: TCMExit); message CM_EXIT;
      procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
      procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
      procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
      procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
      procedure DoClick(Sender: TObject);
   protected
      procedure CreateParams(var Params: TCreateParams); override;
      procedure CreateWnd; override;
		function CanEdit: Boolean; virtual;
		property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
      property Button: TSpeedButton read FButton;
      property BtnFont: TFont read GetFont write SetFont stored StoreFont;
      property BtnCaption: string read GetCaption write SetCaption;
      property BtnGlyph: TBitmap read GetGlyph write SetGlyph;
		property BtnNumGlyphs: TNumGlyphs read GetNumGlyphs write SetNumGlyphs default 1;
		property BtnWidth: integer read GetWidth write SetWidth default -1;
      property Style: TRtComboDlgStyle read FStyle write SetStyle default csEditable;
      property ShowButton: Boolean read GetShowButton write SetShowButton default True;
		property EnableButton: Boolean read GetEnableButton write SetEnableButton default True;
      property OnDialog: TNotifyEvent read FOnDialog write FOnDialog;
   public
      procedure KeyPress(var Key: Char); override;
      procedure KeyDown(var Key: Word; Shift: TShiftState); override;
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
   end;

   TRtComboDlg = class(TRtCustomComboDlg)
   public
      property MaskState;
   published
   	property AutoSize;
      property AutoSelect;
      property Ctl3D;
      property CharCase;
      property Color;
      property DragCursor;
      property DragMode;
      property Enabled;
      property Font;
      property MaxLength;
      property ParentColor;
      property ParentCtl3D;
      property ParentFont;
      property ParentShowHint;
      property PopupMenu;
      property ReadOnly;
      property ShowHint;
      property TabOrder;
      property TabStop;
      property Text;
      property Visible;
      property OnChange;
      property OnClick;
      property OnDblClick;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDrag;
      property OnEnter;
      property OnExit;
      property OnKeyDown;
      property OnKeyPress;
      property OnKeyUp;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
		property Alignment;
      property BtnFont;
      property BtnCaption;
      property BtnGlyph;
		property BtnNumGlyphs;
      property BtnWidth;
      property Style;
      property ShowButton;
		property EnableButton;
      property EditMask;
      property OnDialog;
   end;

procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('RiT',[TRTComboDlg]);
end;

{TRtCustomComboDlg}
constructor TRtCustomComboDlg.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
   FButton := TSpeedButton.Create(self);
   FButton.Parent := self;
   FButton.Height := Height;
   FButton.Top := 0;
   FButton.Width := Height;
   FButton.Font := Font;
   FButton.Left := Width - FButton.Width;
   FButton.Cursor := crArrow;
   FButton.OnClick := DoClick;
   FWidth := -1;
	FStyle := csEditable;
end;

destructor TRtCustomComboDlg.Destroy;
begin
	FButton.Free;
	inherited Destroy;
end;

procedure TRtCustomComboDlg.SetAlignment(Value: TAlignment);
begin
	if FAlignment<>Value then
	begin
		FAlignment := Value;
		RecreateWnd;
	end;
end;

function TRtCustomComboDlg.GetShowButton: Boolean;
begin
	Result := FButton.Width<>0;
end;

procedure TRtCustomComboDlg.SetShowButton(Value: Boolean);
begin
	if (FButton.Width<>0) and (not Value) then
   begin
		FButton.Width := 0;
	   RecreateWnd;
   end
   else if (FButton.Width=0) and (Value) then
   begin
   	if FWidth=-1 then
		  	FButton.Width := Height
      else
		  	FButton.Width := FWidth;
	   RecreateWnd;
   end;
end;

function TRtCustomComboDlg.GetEnableButton: Boolean;
begin
	Result := FButton.Enabled;
end;

procedure TRtCustomComboDlg.SetEnableButton(Value: Boolean);
begin
	if FButton.Enabled<>Value then
		FButton.Enabled := Value;
end;

function TRtCustomComboDlg.GetFont: TFont;
begin
	Result := FButton.Font;
end;

procedure TRtCustomComboDlg.SetFont(Value: TFont);
begin
   FButton.Font := Value;
end;

function TRtCustomComboDlg.StoreFont: Boolean;
begin
	Result := (FButton.Font.Color<>Font.Color) or (FButton.Font.Name<>Font.Name) or
   			 (FButton.Font.Size<>Font.Size) or (FButton.Font.Style<>Font.Style);
end;

function TRtCustomComboDlg.GetCaption: string;
begin
	Result := FButton.Caption;
end;

procedure TRtCustomComboDlg.SetCaption(Value: string);
begin
   FButton.Caption := Value;
end;

function TRtCustomComboDlg.GetGlyph: TBitmap;
begin
	Result := FButton.Glyph;
end;

procedure TRtCustomComboDlg.SetGlyph(Value: TBitmap);
begin
	FButton.Glyph := Value;
end;

function TRtCustomComboDlg.GetNumGlyphs: TNumGlyphs;
begin
	Result := FButton.NumGlyphs;
end;

procedure TRtCustomComboDlg.SetNumGlyphs(Value: TNumGlyphs);
begin
   FButton.NumGlyphs := Value;
end;

function TRtCustomComboDlg.GetWidth: integer;
begin
	Result := FWidth;
end;

procedure TRtCustomComboDlg.SetWidth(Value: integer);
begin
	if (FWidth<>Value) and ((Value>0) or (Value=-1)) then
   begin
   	FWidth := Value;
      Invalidate;
   end;
end;

function TRtCustomComboDlg.CanEdit: Boolean;
begin
	Result := (FStyle=csEditable) and (not ReadOnly);
end;

procedure TRtCustomComboDlg.KeyDown(var Key: Word; Shift: TShiftState);
var
	State: TShiftState;
begin
	inherited KeyDown (Key, Shift);
	{if (not CanEdit) then
   	case Key of
	   	VK_SPACE:FButton.Click;
	   	else		Key := 0;
      end;}
end;

procedure TRtCustomComboDlg.KeyPress(var Key: Char);
begin
   inherited KeyPress(Key);
   if not CanEdit then
		case Key of
      	^C, ^V, ^X, #33..#255:
			begin
				Key := #0;
				MessageBeep(0);
			end;
         #32:	FButton.Click;
      end;
end;

procedure TRtCustomComboDlg.WMPaste (var Message: TMessage);
begin
	if CanEdit then inherited;
end;

procedure TRtCustomComboDlg.WMCut (var Message: TMessage);
begin
	if CanEdit then inherited;
end;

procedure TRtCustomComboDlg.DoClick(Sender: TObject);
begin
  	SetFocus;
	if assigned(FOnDialog) then FOnDialog(Self);
end;

procedure TRtCustomComboDlg.CreateParams(var Params: TCreateParams);
const
	Alignments: array[TAlignment] of Longint = (ES_LEFT, ES_RIGHT, ES_CENTER);
begin
	inherited CreateParams(Params);
	Params.Style := Params.Style or ES_MULTILINE or Alignments[FAlignment] or WS_CLIPCHILDREN;
end;

procedure TRtCustomComboDlg.CreateWnd;
begin
	inherited CreateWnd;
	SetEditRect;
end;

procedure TRtCustomComboDlg.SetEditRect;
var
	Loc: TRect;
begin
	SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
	Loc.Bottom := ClientHeight+1;
	Loc.Right := FButton.Left-2;
	Loc.Top := 0;
	Loc.Left := 0;
	SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
	{SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));}
end;

function TRtCustomComboDlg.GetMinHeight: integer;
var
	DC: HDC;
	SaveFont: HFont;
	I: Integer;
	SysMetrics, Metrics: TTextMetric;
begin
	DC := GetDC(0);
	GetTextMetrics(DC, SysMetrics);
	SaveFont := SelectObject(DC, Font.Handle);
	GetTextMetrics(DC, Metrics);
	SelectObject(DC, SaveFont);
	ReleaseDC(0, DC);
	I := SysMetrics.tmHeight;
	if I > Metrics.tmHeight then I := Metrics.tmHeight;
	Result := Metrics.tmHeight + I div 4 + GetSystemMetrics(SM_CYBORDER) * 4+1;
end;

procedure TRtCustomComboDlg.WMSize(var Message: TWMSize);
var
	M,W: integer;
begin
	inherited;
   M := GetMinHeight;
   if Height<M then
   	Height := M
	else if FButton<>nil then
	begin
   	if FWidth=-1 then
      	W := Height
      else
      	W := FWidth;
   	if not NewStyleControls then
			FButton.SetBounds(Width-W,0,W,Height)
      else if FWidth=-1 then
			FButton.SetBounds(Width-W+2,2,W-4,ClientHeight-4)
      else
			FButton.SetBounds(Width-W-2,2,W,ClientHeight-4);
		SetEditRect;
	end;
end;

procedure TRtCustomComboDlg.WMPaint(var Message: TWMPaint);
begin
	FButton.Invalidate;
	inherited;
end;

procedure TRtCustomComboDlg.CMEnabledChanged(var Message: TMessage);
begin
	inherited;
	FButton.Enabled := Enabled;
end;

procedure TRtCustomComboDlg.SetStyle(Value: TRtComboDlgStyle);
begin
	if FStyle <> Value then
		FStyle := Value;
end;

procedure TRtCustomComboDlg.WMSetFocus(var Message: TWMSetFocus);
begin
	inherited;
	if not CanEdit then
		HideCaret(Handle);
end;

procedure TRtCustomComboDlg.CMExit(var Message: TCMExit);
begin
	inherited;
	if not CanEdit then Invalidate;
end;

procedure TRtCustomComboDlg.CMEnter(var Message: TCMGotFocus);
begin
	inherited;
   if (FStyle=csEditable) and (not FButton.Visible) then
   	FButton.Click;
end;

end.
